import 'package:flutter/material.dart';
import 'package:jobportal/services/global_variables.dart';
import 'package:jobportal/UI/bottom_navigation_bar/jobseeker_bottom_navigation_bar.dart';
import 'package:jobportal/data/company_job.dart';
import 'package:jobportal/data/job.dart';
import 'package:jobportal/logic/fetch.dart';
import 'package:jobportal/widgets/company_job_widget.dart';
import '../../widgets/jobwidget.dart';
import '../bottom_navigation_bar/company_bottom_navigation_bar.dart';
import '../notification/notification_screen.dart';

class HomeScreen extends StatefulWidget {
// Add the job to the list
  const HomeScreen({super.key});

  @override
  State<HomeScreen> createState() => _HomeScreenState();
}

class _HomeScreenState extends State<HomeScreen> {
  final TextEditingController _search = TextEditingController();
  String searchQuery = "Search query";
  int x = 0;
  late int pageNumber;
  late int pageSearchNumber;
  late int pageFilterNumber;

  late bool isPagenationForSearch;
  late bool isPagenationForFilter;

  List<Job> jobs = [];
  List<CompanyJob> companyJobs = [];
  List<CompanyJob> allCompanyJobs = [];
  List<Job> allJobseekerJobs = [];

  @override
  void initState() {
    super.initState();
    pageNumber = 0;
    pageFilterNumber = 0;
    pageSearchNumber = 0;
    isPagenationForSearch = false;
    isPagenationForFilter = false;
    fetchAllJobs();
    identifier.iscompany == true
        ? fetchDataForCompany()
        : fetchDataForJobseeker();
  }

  void _incrementPage() {
    setState(() {
      pageNumber++;
      fetchAllJobs();
    });
  }

  void _decrementPage() {
    setState(() {
      if (pageNumber >= 1) {
        pageNumber--;
        fetchAllJobs();
      }
    });
  }

  void _incrementSearchPage() {
    setState(() {
      pageSearchNumber++;
      _updateSearch(_search.text);
    });
  }

  void _decrementSearchPage() {
    setState(() {
      if (pageSearchNumber >= 1) {
        pageSearchNumber--;
        _updateSearch(_search.text);
      }
    });
  }

  void fetchDataForCompany() async {
    x = await Fetch.getUnReadCompanyNotificationsNumber();
    setState(() {});
  }

  void fetchDataForJobseeker() async {
    x = await Fetch.getUnReadJobseekerNotificationsNumber();
    setState(() {});
  }

  void fetchAllJobs() async {
    identifier.iscompany == false
        ? {
            jobs = await Fetch.getAllJobsByPage(pageNumber),
            allJobseekerJobs = jobs
          }
        : {
            companyJobs = await Fetch.getJobsByCompanyIdAndPage(
                identifier.id, pageNumber),
            allCompanyJobs = companyJobs
          };
    setState(() {});
  }

  Widget _searchField() {
    return TextField(
      controller: _search,
      autocorrect: true,
      decoration: const InputDecoration(
          border: InputBorder.none,
          hintText: "Search for jobs....",
          hintStyle: TextStyle(color: Colors.white)),
      style: const TextStyle(color: Colors.white, fontSize: 16.0),
    );
  }

  void _clearSearch() {
    setState(() {
      _search.clear();
      _updateSearch("");
    });
  }

  Future<void> _updateSearch(String newQuery) async {
    setState(() {
      searchQuery = newQuery;
    });

    if (newQuery.isEmpty) {
      setState(() {
        isPagenationForSearch = false;
        companyJobs = allCompanyJobs;
      });
    } else {
      final updatedCompanyJobs =
          await Fetch.getJobsByCompanyIdAndJobNameAndPageNumber(
              identifier.id, newQuery, pageSearchNumber);
      setState(() {
        isPagenationForSearch = true;
        companyJobs = updatedCompanyJobs;
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      child: SafeArea(
          child: Scaffold(
              bottomNavigationBar: identifier.iscompany == true
                  ? MyBottomNavigationBar(
                      indexNum: 0,
                    )
                  : JobseekerBottomNavigationBar(
                      indexNum: 0,
                    ),
              backgroundColor: Colors.grey.shade300,
              drawer: const Drawer(),
              appBar: AppBar(
                backgroundColor: Colors.orange,
                iconTheme: const IconThemeData(color: Colors.black),
                title: identifier.iscompany == true
                    ? _searchField()
                    : const SizedBox(),
                actions: [
                  identifier.iscompany == true
                      ? IconButton(
                          onPressed: () {
                            _clearSearch();
                          },
                          icon: const Icon(Icons.clear))
                      : const SizedBox(),
                  identifier.iscompany == true
                      ? IconButton(
                          onPressed: () {
                            String searchQuery = _search.text;
                            _updateSearch(searchQuery);
                          },
                          icon: const Icon(Icons.search))
                      : const SizedBox(),
                  identifier.iscompany == false
                      ? IconButton(
                          onPressed: () {
                            _showFilterDialog();
                          },
                          icon: const Icon(Icons.filter_list))
                      : const SizedBox(),
                  InkWell(
                      onTap: () async {
                        await Navigator.push(
                          context,
                          MaterialPageRoute(
                            builder: (context) =>
                                const ShowMyNotificationScreen(),
                          ),
                        );
                        setState(() {
                          identifier.iscompany == true
                              ? fetchDataForCompany()
                              : fetchDataForJobseeker();
                        });
                      },
                      child: Stack(children: [
                        const Padding(
                          padding: EdgeInsets.all(8.0),
                          child: Icon(
                            Icons.notifications,
                            size: 40,
                            color: Colors.black87,
                          ),
                        ),
                        x > 0
                            ? Positioned(
                                right: 8,
                                bottom: 12,
                                child: Container(
                                  width: 17,
                                  height: 17,
                                  decoration: const BoxDecoration(
                                    color: Colors.red,
                                    shape: BoxShape.circle,
                                  ),
                                  child: Padding(
                                    padding: const EdgeInsets.all(1.0),
                                    child: Text(
                                      x.toString(),
                                      textAlign: TextAlign.center,
                                      style: const TextStyle(
                                          color: Colors.white, fontSize: 9),
                                    ),
                                  ),
                                ))
                            : const SizedBox.shrink()
                      ])),
                ],
              ),
              body: Stack(
                children: [
                  identifier.iscompany == false
                      ? ListView.builder(
                          itemCount: jobs.length,
                          itemBuilder: (context, index) {
                            return ShowJobWidget(job: jobs[index]);
                          },
                        )
                      : ListView.builder(
                          itemCount: companyJobs.length,
                          itemBuilder: (context, index) {
                            return ShowCompanyJobWidget(
                                job: companyJobs[index]);
                          },
                        ),
                  // (identifier.iscompany == true)
                  //     ? isPagenationForSearch == false
                  //         ? Positioned(
                  //             bottom: 16,
                  //             left: 0,
                  //             right: 0,
                  //             child: Row(
                  //               mainAxisAlignment: MainAxisAlignment.center,
                  //               children: [
                  //                 IconButton(
                  //                   onPressed: _decrementPage,
                  //                   icon: const Icon(
                  //                     Icons.arrow_left,
                  //                     color: Colors.red,
                  //                     size: 40,
                  //                   ),
                  //                 ),
                  //                 Padding(
                  //                   padding: const EdgeInsets.symmetric(
                  //                       horizontal: 16.0),
                  //                   child: Text(
                  //                     'Page ${pageNumber + 1} ',
                  //                     style: const TextStyle(
                  //                         fontWeight: FontWeight.bold),
                  //                   ),
                  //                 ),
                  //                 IconButton(
                  //                   onPressed: _incrementPage,
                  //                   icon: const Icon(
                  //                     Icons.arrow_right,
                  //                     color: Colors.green,
                  //                     size: 40,
                  //                   ),
                  //                 ),
                  //               ],
                  //             ),
                  //           )
                  //         : Positioned(
                  //             bottom: 16,
                  //             left: 0,
                  //             right: 0,
                  //             child: Row(
                  //               mainAxisAlignment: MainAxisAlignment.center,
                  //               children: [
                  //                 IconButton(
                  //                   onPressed: _decrementSearchPage,
                  //                   icon: const Icon(
                  //                     Icons.arrow_left,
                  //                     color: Colors.black,
                  //                     size: 40,
                  //                   ),
                  //                 ),
                  //                 Padding(
                  //                   padding: const EdgeInsets.symmetric(
                  //                       horizontal: 16.0),
                  //                   child: Text(
                  //                     'Page ${pageSearchNumber + 1} ',
                  //                     style: const TextStyle(
                  //                         fontWeight: FontWeight.bold),
                  //                   ),
                  //                 ),
                  //                 IconButton(
                  //                   onPressed: _incrementSearchPage,
                  //                   icon: const Icon(
                  //                     Icons.arrow_right,
                  //                     color: Colors.black,
                  //                     size: 40,
                  //                   ),
                  //                 ),
                  //               ],
                  //             ),
                  //           )
                  //     : isPagenationForFilter == false
                  //         ? Positioned(
                  //             bottom: 16,
                  //             left: 0,
                  //             right: 0,
                  //             child: Row(
                  //               mainAxisAlignment: MainAxisAlignment.center,
                  //               children: [
                  //                 IconButton(
                  //                   onPressed: _decrementPage,
                  //                   icon: const Icon(
                  //                     Icons.arrow_left,
                  //                     color: Colors.red,
                  //                     size: 40,
                  //                   ),
                  //                 ),
                  //                 Padding(
                  //                   padding: const EdgeInsets.symmetric(
                  //                       horizontal: 16.0),
                  //                   child: Text(
                  //                     'Page ${pageNumber + 1} ',
                  //                     style: const TextStyle(
                  //                         fontWeight: FontWeight.bold),
                  //                   ),
                  //                 ),
                  //                 IconButton(
                  //                   onPressed: _incrementPage,
                  //                   icon: const Icon(
                  //                     Icons.arrow_right,
                  //                     color: Colors.green,
                  //                     size: 40,
                  //                   ),
                  //                 ),
                  //               ],
                  //             ),
                  //           )
                  //         : const SizedBox()
                ],
              ))),
    );
  }

  void _showFilterDialog() {
    showDialog(
      context: context,
      builder: (BuildContext context) {
        return AlertDialog(
            title: const Text('Filter Jobs'),
            content: Column(
              mainAxisSize: MainAxisSize.min,
              children: [
                ListTile(
                  title: const Text('Filter by Specification Name'),
                  onTap: () {
                    Navigator.of(context).pop();
                    _filterBySpecificationName();
                  },
                ),
                const SizedBox(
                  height: 8.0,
                  child: Divider(
                    thickness: 1,
                  ),
                ),
                ListTile(
                  title: const Text('Filter by Time Type'),
                  onTap: () {
                    Navigator.of(context).pop();
                    _filterByTimeType();
                  },
                ),
                const SizedBox(
                  height: 8.0,
                  child: Divider(
                    thickness: 1,
                  ),
                ),
                ListTile(
                  title: const Text('Show All Jobs'),
                  onTap: () {
                    Navigator.of(context).pop();
                    setState(() {
                      isPagenationForFilter = false;

                      jobs = allJobseekerJobs;
                    });
                  },
                ),
              ],
            ),
            actions: [
              TextButton(
                onPressed: () {
                  Navigator.of(context).pop();
                },
                child: const Text('Cancel'),
              ),
            ]);
      },
    );
  }

  Future<void> _getJobsBySpecificationName(String specificationName) async {
    jobs = await Fetch.getAllJobsBySpecificationName(specificationName);
    setState(() {
      isPagenationForFilter = true;
    });
  }

  Future<void> _getJobsByTimeType(String timeType) async {
    jobs = await Fetch.getAllJobsByTimeType(timeType);
    setState(() {
      isPagenationForFilter = true;
    });
  }

  void _filterBySpecificationName() {
    TextEditingController _specificationNameController =
        TextEditingController();
    showDialog(
      context: context,
      builder: (BuildContext context) {
        return AlertDialog(
          title: const Text('Filter by Specification Name'),
          content: TextField(
            controller: _specificationNameController,
            decoration: const InputDecoration(
              hintText: 'Enter specification name',
            ),
          ),
          actions: [
            TextButton(
              onPressed: () {
                Navigator.of(context).pop();
              },
              child: const Text('Cancel'),
            ),
            TextButton(
              onPressed: () {
                String specificationName = _specificationNameController.text;
                _getJobsBySpecificationName(specificationName);
                Navigator.of(context).pop();
              },
              child: const Text('Filter'),
            ),
          ],
        );
      },
    );
  }

  void _filterByTimeType() {
    showDialog(
      context: context,
      builder: (BuildContext context) {
        return AlertDialog(
          title: const Text('Filter by Time Type'),
          content: Column(
            mainAxisSize: MainAxisSize.min,
            children: [
              ListTile(
                title: const Text('Part-time and On-site Work'),
                onTap: () {
                  _getJobsByTimeType('Part-time and On-site Work');

                  Navigator.of(context).pop();
                },
              ),
              ListTile(
                title: const Text('Full-time and On-site Work'),
                onTap: () {
                  _getJobsByTimeType('Full-time and On-site Work');

                  Navigator.of(context).pop();
                },
              ),
              ListTile(
                title: const Text('Part-time and Remotly Work'),
                onTap: () {
                  _getJobsByTimeType('Part-time and Remotly Work');

                  Navigator.of(context).pop();
                },
              ),
              ListTile(
                title: const Text('Full-time and Remotly Work'),
                onTap: () {
                  _getJobsByTimeType('Full-time and Remotly Work');

                  Navigator.of(context).pop();
                },
              ),
            ],
          ),
          actions: [
            TextButton(
              onPressed: () {
                Navigator.of(context).pop();
              },
              child: const Text('Cancel'),
            ),
          ],
        );
      },
    );
  }
}
